use std::{collections::VecDeque, fs::File};
use time::OffsetDateTime;

use rss::{ChannelBuilder, Item, ItemBuilder};

#[derive(Clone, PartialEq, Eq)]
struct RemainingPlace {
    id: String,
    description: String,
    date: String,
    free: usize,
}

fn main() -> Result<(), reqwest::Error> {
    // const URL: &str = "https://www.lfs.sachsen.de/restplatzboerse-5152.html";
    const URL: &str = "http://127.0.0.1:8080/tip.html";

    let mut channel = ChannelBuilder::default()
        .title(String::from("LFS Restplatzbörse"))
        .link(URL.to_string())
        .description(String::from("Ein RSS Feed der Restplatzbörse der Landesfeuerwehrschule Sachsen. Nicht offiziell."))
        .language(Some("de-DE".to_string()))
        .build();

    let mut last_places: Vec<RemainingPlace> = Vec::new();
    let mut items: VecDeque<Item> = VecDeque::new();

    loop {
        let body = reqwest::blocking::get(URL).unwrap().text().unwrap();

        let start = body.find("<tbody").unwrap();
        let end = body.find("</tbody>").unwrap();

        let table = &body[start..=(end + 7)];

        let mut places: Vec<RemainingPlace> = Vec::new();
        let mut iter = table.lines();

        while let Some(line) = iter.next() {
            if line.contains("<tr>") {
                let id = parse_node(iter.next().unwrap());
                let description = parse_node(iter.next().unwrap());
                let date = parse_node(iter.next().unwrap());
                let free = parse_node(iter.next().unwrap()).parse().unwrap();

                let place = RemainingPlace {
                    id,
                    description,
                    date,
                    free,
                };
                places.push(place);
            }
        }

        let condition = places.len() == last_places.len()
            && places
                .iter()
                .zip(last_places.iter())
                .all(|(one, two)| one == two);

        if !condition {
            last_places = places.clone();

            let title = format!("Restplatzbörse Update - {}", OffsetDateTime::now_utc());
            let content = places
                .iter()
                .map(|place| {
                    format!(
                        "{} - {} - {} - {}",
                        place.id, place.description, place.date, place.free
                    )
                })
                .collect::<Vec<String>>()
                .join("\n");

            let item = ItemBuilder::default()
                .title(Some(title))
                .content(Some(content))
                .build();

            items.push_back(item);

            if items.len() > 10 {
                items.pop_front().unwrap();
            }

            channel.set_items(items.clone());

            let output = File::create("rss.txt").unwrap();
            channel.pretty_write_to(output, ' ' as u8, 2).unwrap();

            println!("Got Update and wrote to file");
        } else {
            println!("No update");
        }
        std::thread::sleep(std::time::Duration::from_secs_f64(60.0));
    }
}

fn parse_node(input: &str) -> String {
    let start = input.find(">").unwrap();
    let end = input.find("</").unwrap();

    input[(start + 1)..end].to_string()
}
